# Tile Grid.rb
# v1.0.1
# Trog Luddite
# March 2013

# Adds an item 'Tile Grid' to the 'Draw' menu.
# Creates a simple regular line grid containing squares of size 'Tile Size'
# separated from each other by a spacing of 'Grout Width'
# Grid is always placed on the ground plane at the origin with tiles extending
# in the positive green and red directions.
# Grid is created as a group.
# Measurements are in current model units.
# If 'Grout Width' = zero, a simple grid of squares is made.

# No warranty is implied - use this plugin at your own risk, the author
# assumed no responsibility for unforseen consequences of its use.
# Free to use, copy, distrbute and copy, so long as the code is always
# attributed to Trog Luddite (S.Munns)

# Version history
# v1.0.1 -Invalid Group.definition method removed (thanks thomthom and sdmitch)
#        -Exception handling added to catch invalid inputs (thanks Dan Rathburn)

module Trog
 
  def self.add_tile_grid
  
    mod = Sketchup.active_model
  
    # Get user input values
    prompts = ['Tile Size', 'Grout Width', 'Tile Count X', 'Tile Count Y']
    defaults = [1.0.to_l, 0.0.to_l, 10, 10]
    title = "Enter tile grid values"
    vals = UI.inputbox(prompts,defaults,title)
    return nil unless vals
    tile, grout, x_count, y_count = vals
  
    mod.start_operation("Add Tile Grid",true)
    
    # Make group - named for the tile and grout sizes for easy ID
    ents = mod.entities
    group = ents.add_group
    group.name = "Tile Grid #{tile}:#{grout}"
    grp_ents = group.entities
    
    # Parameters
    step = tile + grout
    x_count = x_count.to_i
    y_count = y_count.to_i
    x_size = x_count * step + grout
    y_size = y_count * step + grout
     
    # Add lines, x direction
    (0..x_count).each do |x|
      pt1 = Geom::Point3d.new(x * step, 0, 0)
      pt2 = Geom::Point3d.new(x * step, y_size, 0)
      grp_ents.add_line(pt1,pt2)
      unless grout == 0.0
        pt1 = Geom::Point3d.new(x * step + grout, 0, 0)
        pt2 = Geom::Point3d.new(x * step + grout, y_size, 0)
        grp_ents.add_line(pt1,pt2)
      end
    end
    
    # Add lines, y direction
    (0..y_count).each do |y|
      pt1 = Geom::Point3d.new(0, y * step, 0)
      pt2 = Geom::Point3d.new(x_size, y * step, 0)
      grp_ents.add_line(pt1,pt2)
      unless grout == 0.0
        pt1 = Geom::Point3d.new(0, y * step + grout, 0)
        pt2 = Geom::Point3d.new(x_size, y * step + grout, 0)
        grp_ents.add_line(pt1,pt2)
      end
    end
    
    mod.commit_operation
    
  # Catch exceptions  
  rescue => e
    mod.abort_operation
    puts("error #<#{e.class.name}:#{e.message}.>")
    puts(e.backtrace) if $VERBOSE
  end
  
  # Add menu item
  unless file_loaded?("Tile Grid.rb")
    menu = UI.menu("Draw")
    menu.add_item("Tile Grid"){self.add_tile_grid}
    file_loaded("Tile Grid.rb")
  end
  
end
